#include "../include/gray_buffer.h"


//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
guint8 * gray_buffer::getpixel(const GdkPixbuf * pixbuf, const guint &x, const guint &y)
{
      return gdk_pixbuf_get_pixels(pixbuf) + gdk_pixbuf_get_rowstride(pixbuf) * y + gdk_pixbuf_get_n_channels(pixbuf) * x;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
guint8 gray_buffer::gray_rgb32(const guint8 *pix)
{
#if G_BYTE_ORDER == G_LITTLE_ENDIAN
                 return guint8((pix[0]+pix[1]+pix[2])/3);
#else
                 return guint8((pix[1]+pix[2]+pix[3])/3);
#endif
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
void gray_buffer::charger_pixbuf(const GdkPixbuf * pixbuf)
{
     if (buffer != NULL) 
           delete[]  buffer;
  
     width = gdk_pixbuf_get_width(pixbuf);
     height = gdk_pixbuf_get_height(pixbuf);
     buffer = new guint8[width*height];
     
 
     for(guint j = 0; j < height; j++)
        for(guint i = 0; i < width; i++)
           buffer[i+j*width] = gray_rgb32(getpixel(pixbuf,i,j));
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
gray_buffer::gray_buffer(const GdkPixbuf * pixbuf)
{
     charger_pixbuf(pixbuf);
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
gray_buffer::gray_buffer()
{
      width = 0;
      height = 0;
      buffer = NULL;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
gray_buffer::~gray_buffer()
{
     delete[]  buffer;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
guint8 gray_buffer::get_pixel(const guint &x, const guint &y)
{
     return buffer[x+y*width];
     
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
guint8 * gray_buffer::get_pixels()
{
     return buffer;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
guint gray_buffer::get_width()
{
     return width;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
guint gray_buffer::get_height()
{
     return height;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>




//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
int gray_buffer::coul_requent(int &coul, guint8 nb_coul)
{
       return int(round( round(coul * (nb_coul) / 255) *255 / (nb_coul)) );
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>




//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
void gray_buffer::put_pixel(const guint &x, const guint &y, guint8 &pix)
{
     buffer[x+y*width] = pix;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
void gray_buffer::put_pixel_verif_pix(const guint x, const guint y, guint pix)
{
     if(pix < 0)
     {
         buffer[x+y*width] = 0;
         return;
     }
     
     if(pix > 255)
     {
         buffer[x+y*width] = 255;
         return;
     }
      
     buffer[x+y*width] = pix;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
void gray_buffer::ecrire_erreur(guint &x, guint &y, int erreur)
{
    if(!erreur)
       return;
       
    if (y+1 < height)
    {
        if (x-1 > 0)
           put_pixel_verif_pix( x-1, y+1, get_pixel( x-1, y+1) - (int)round(erreur * 3/16) );
        (3);
        put_pixel_verif_pix( x, y+1, get_pixel( x, y+1) - (int)round(erreur * 5/16) );
        
        if (x+1 < width)
           put_pixel_verif_pix( x+1, y+1, get_pixel( x+1, y+1) - (int)round(erreur * 1/16) );
    }
    
    if (x+1 < width)
       put_pixel_verif_pix( x+1, y, get_pixel( x+1, y) - (int)round(erreur * 7/16) );
    
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>






//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
guint gray_buffer::reduire_nb_couleurs(int nb_color, bool diff_err)
{
    // si diff_err == 1, la diffusion d'erreur Floyd-Steinberg
    
    guint8 pal_conv[256];
    nb_color--;
    for(int i = 0; i < 256; i++)
       pal_conv[i] = coul_requent(i, guint8(nb_color));
    
    
    guint8 pix;
    guint8 pix_req;
    
    if(diff_err)
     {
        for(guint y = 0; y < height; y++)
          for(guint x = 0; x < width; x++)
            {
                pix = get_pixel( x, y );
                pix_req = pal_conv[pix];
                put_pixel( x, y, pix_req );
                ecrire_erreur( x, y, pix_req - pix);
            }
     }   
     else 
     {
        for(guint y = 0; y < height; y++)
          for(guint x = 0; x < width; x++)
            {
                pix = get_pixel( x, y );
                pix_req = pal_conv[pix];
                put_pixel( x, y, pix_req );
            }
     }    
    
    
     return 0;
}
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

